@@ -5,46 +5,23 @@
  *                            | (__| |_| |  _ <| |___ 
  *                             \___|\___/|_| \_\_____|
  *
- *  The contents of this file are subject to the Mozilla Public License
- *  Version 1.0 (the "License"); you may not use this file except in
- *  compliance with the License. You may obtain a copy of the License at
- *  http://www.mozilla.org/MPL/
+ * Copyright (C) 2000, Daniel Stenberg, <daniel@haxx.se>, et al.
  *
- *  Software distributed under the License is distributed on an "AS IS"
- *  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
- *  License for the specific language governing rights and limitations
- *  under the License.
+ * In order to be useful for every potential user, curl and libcurl are
+ * dual-licensed under the MPL and the MIT/X-derivate licenses.
  *
- *  The Original Code is Curl.
+ * You may opt to use, copy, modify, merge, publish, distribute and/or sell
+ * copies of the Software, and permit persons to whom the Software is
+ * furnished to do so, under the terms of the MPL or the MIT/X-derivate
+ * licenses. You may pick one of these licenses.
  *
- *  The Initial Developer of the Original Code is Daniel Stenberg.
+ * This software is distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY
+ * KIND, either express or implied.
  *
- *  Portions created by the Initial Developer are Copyright (C) 1998.
- *  All Rights Reserved.
- *
- * ------------------------------------------------------------
- * Main author:
- * - Daniel Stenberg <Daniel.Stenberg@haxx.nu>
- *
- * 	http://curl.haxx.nu
- *
- * $Source: /cvs/curl/curl/lib/telnet.c,v $
- * $Revision: 1.8 $
- * $Date: 1999/09/06 06:59:41 $
- * $Author: dast $
- * $State: Exp $
- * $Locker:  $
- *
- * ------------------------------------------------------------
- *
- * This implementation of the TELNET protocol is written by
- * Linus Nielsen <Linus.Nielsen@haxx.nu>,
- * with some code snippets stolen from the BSD Telnet client.
- *
- * The negotiation is performed according to RFC 1143 (D. Bernstein,
- * "The Q Method of Implementing TELNET Option Negotiation")
- *
- ****************************************************************************/
+ * $Id: telnet.c,v 1.11 2001/01/25 12:22:17 bagder Exp $
+ *****************************************************************************/
+
+#include "setup.h"
 
 /* -- WIN32 approved -- */
 #include <stdio.h>
@@ -57,8 +34,6 @@
 
 #include <errno.h>
 
-#include "../include/setup.h"
-
 #if defined(WIN32) && !defined(__GNUC__) || defined(__MINGW32__)
 #include <winsock.h>
 #include <time.h>
@@ -74,8 +49,12 @@
 #include <unistd.h>
 #endif
 #include <netdb.h>
+#ifdef HAVE_ARPA_INET_H
 #include <arpa/inet.h>
+#endif
+#ifdef HAVE_NET_IF_H
 #include <net/if.h>
+#endif
 #include <sys/ioctl.h>
 #include <signal.h>
 
@@ -91,18 +70,15 @@
 #endif
 
 #include "urldata.h"
-#include "../include/curl.h"
-#include "download.h"
+#include <curl/curl.h>
+#include "transfer.h"
 #include "sendf.h"
-#include "formdata.h"
-#include "progress.h"
 
 #define _MPRINTF_REPLACE /* use our functions only */
-#include "../include/mprintf.h"
+#include <curl/mprintf.h>
 
 #define  TELOPTS
 #define  TELCMDS
-#define  SLC_NAMES
 
 #include "arpa_telnet.h"
 
@@ -119,10 +95,7 @@
 #define  SB_EOF() (subpointer >= subend)
 #define  SB_LEN() (subend - subpointer)
 
-void telwrite(struct UrlData *data,
-	      unsigned char *buffer,	/* Data to write */
-	      int count);		/* Number of bytes to write */
-
+static
 void telrcv(struct UrlData *data,
 	    unsigned char *inbuf,	/* Data received from socket */
 	    int count);			/* Number of bytes received */
@@ -176,6 +149,7 @@ static int him[256];
 static int himq[256]; 
 static int him_preferred[256]; 
 
+static
 void init_telnet(struct UrlData *data)
 {
    telrcv_state = TS_DATA;
@@ -221,7 +195,7 @@ static void printoption(struct UrlData *data,
    char *fmt;
    char *opt;
    
-   if (data->conf & CONF_VERBOSE)
+   if (data->bits.verbose)
    {
       if (cmd == IAC)
       {
@@ -267,6 +241,7 @@ static void send_negotiation(struct UrlData *data, int cmd, int option)
    printoption(data, "SENT", cmd, option);
 }
 
+static
 void set_remote_option(struct UrlData *data, int option, int newstate)
 {
    if(newstate == YES)
@@ -347,6 +322,7 @@ void set_remote_option(struct UrlData *data, int option, int newstate)
    }
 }
 
+static
 void rec_will(struct UrlData *data, int option)
 {
    switch(him[option])
@@ -398,6 +374,7 @@ void rec_will(struct UrlData *data, int option)
    }
 }
    
+static
 void rec_wont(struct UrlData *data, int option)
 {
    switch(him[option])
@@ -521,6 +498,7 @@ void set_local_option(struct UrlData *data, int option, int newstate)
    }
 }
 
+static
 void rec_do(struct UrlData *data, int option)
 {
    switch(us[option])
@@ -571,7 +549,8 @@ void rec_do(struct UrlData *data, int option)
       break;
    }
 }
-   
+
+static   
 void rec_dont(struct UrlData *data, int option)
 {
    switch(us[option])
@@ -624,7 +603,7 @@ static void printsub(struct UrlData *data,
 {
    int i = 0;
 
-   if (data->conf & CONF_VERBOSE)
+   if (data->bits.verbose)
    {
       if (direction)
       {
@@ -690,6 +669,7 @@ static void suboption(struct UrlData *data)
    return;
 }
 
+static
 void telrcv(struct UrlData *data,
 	    unsigned char *inbuf,	/* Data received from socket */
 	    int count)			/* Number of bytes received */
@@ -709,8 +689,8 @@ void telrcv(struct UrlData *data,
 	 {
 	    break;   /* Ignore \0 after CR */
 	 }
-	 
-	 data->fwrite((char *)&c, 1, 1, data->out);
+
+	 Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
 	 continue;
 
       case TS_DATA:
@@ -724,7 +704,7 @@ void telrcv(struct UrlData *data,
 	    telrcv_state = TS_CR;
 	 }
 
-	 data->fwrite((char *)&c, 1, 1, data->out);
+	 Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
 	 continue;
 
       case TS_IAC:
@@ -748,8 +728,8 @@ void telrcv(struct UrlData *data,
 	   telrcv_state = TS_SB;
 	   continue;
 	case IAC:
-	   data->fwrite((char *)&c, 1, 1, data->out);
-	   break;
+          Curl_client_write(data, CLIENTWRITE_BODY, (char *)&c, 1);
+          break;
 	case DM:
 	case NOP:
 	case GA:
@@ -839,95 +819,73 @@ void telrcv(struct UrlData *data,
    }
 }
 
-void telwrite(struct UrlData *data,
-	      unsigned char *buffer,	/* Data to write */
-	      int count)		/* Number of bytes to write */
+CURLcode Curl_telnet_done(struct connectdata *conn)
 {
-   unsigned char outbuf[2];
-   int out_count = 0;
-   int bytes_written;
-
-   while(count--)
-   {
-      outbuf[0] = *buffer++;
-      out_count = 1;
-      if(outbuf[0] == IAC)
-	 outbuf[out_count++] = IAC;
-      
-#ifndef USE_SSLEAY
-      bytes_written = swrite(data->firstsocket, outbuf, out_count);
-#else
-      if (data->use_ssl) {
-        bytes_written = SSL_write(data->ssl, (char *)outbuf, out_count);
-      }
-      else {
-        bytes_written = swrite(data->firstsocket, outbuf, out_count);
-      }
-#endif /* USE_SSLEAY */
-   }
+  return CURLE_OK;
 }
 
-UrgError telnet(struct UrlData *data)
+CURLcode Curl_telnet(struct connectdata *conn)
 {
-   int sockfd = data->firstsocket;
-   fd_set readfd;
-   fd_set keepfd;
+  struct UrlData *data = conn->data;
+  int sockfd = data->firstsocket;
+  fd_set readfd;
+  fd_set keepfd;
 
-   bool keepon = TRUE;
-   char *buf = data->buffer;
-   int nread;
+  bool keepon = TRUE;
+  char *buf = data->buffer;
+  size_t nread;
 
-   init_telnet(data);
+  init_telnet(data);
    
-   FD_ZERO (&readfd);		/* clear it */
-   FD_SET (sockfd, &readfd);
-   FD_SET (1, &readfd);
+  FD_ZERO (&readfd);		/* clear it */
+  FD_SET (sockfd, &readfd);
+  FD_SET (1, &readfd);
 
-   keepfd = readfd;
+  keepfd = readfd;
 
-   while (keepon)
-   {
-      readfd = keepfd;		/* set this every lap in the loop */
+  while (keepon) {
+    readfd = keepfd;		/* set this every lap in the loop */
 
-      switch (select (sockfd + 1, &readfd, NULL, NULL, NULL))
-      {
-      case -1:			/* error, stop reading */
-	 keepon = FALSE;
-	 continue;
-      case 0:			/* timeout */
-	 break;
-      default:			/* read! */
-	 if(FD_ISSET(1, &readfd))
-	 {
-	    nread = read(1, buf, 255);
-	    telwrite(data, (unsigned char *)buf, nread);
-	 }
+    switch (select (sockfd + 1, &readfd, NULL, NULL, NULL)) {
+    case -1:			/* error, stop reading */
+      keepon = FALSE;
+      continue;
+    case 0:			/* timeout */
+      break;
+    default:			/* read! */
+      if(FD_ISSET(1, &readfd)) { /* read from stdin */
+        unsigned char outbuf[2];
+        int out_count = 0;
+        size_t bytes_written;
+        char *buffer = buf;
+        
+        nread = read(1, buf, 255);
 
-	 if(FD_ISSET(sockfd, &readfd))
-	 {
-#ifndef USE_SSLEAY
-	    nread = sread (sockfd, buf, BUFSIZE - 1);
-#else
-	    if (data->use_ssl) {
-	       nread = SSL_read (data->ssl, buf, BUFSIZE - 1);
-	    }
-	    else {
-	       nread = sread (sockfd, buf, BUFSIZE - 1);
-	    }
-#endif /* USE_SSLEAY */
-	 }
-
-	 /* if we receive 0 or less here, the server closed the connection and
-	   we bail out from this! */
-	if (nread <= 0) {
-	  keepon = FALSE;
-	  break;
-	}
-
-	 telrcv(data, (unsigned char *)buf, nread);
+        while(nread--) {
+          outbuf[0] = *buffer++;
+          out_count = 1;
+          if(outbuf[0] == IAC)
+            outbuf[out_count++] = IAC;
+      
+          Curl_write(conn, data->firstsocket, outbuf,
+                     out_count, &bytes_written);
+        }
       }
-   }
-   return URG_OK;
+
+      if(FD_ISSET(sockfd, &readfd))
+        Curl_read(conn, sockfd, buf, BUFSIZE - 1, &nread);
+
+      /* if we receive 0 or less here, the server closed the connection and
+         we bail out from this! */
+      if (nread <= 0) {
+        keepon = FALSE;
+        break;
+      }
+
+      telrcv(data, (unsigned char *)buf, nread);
+    }
+  }
+  return CURLE_OK;
 }
 
 
